@@ -49,7 +49,6 @@
 #include <arpa/inet.h>
 #include <linux/types.h>
 #include <linux/sockios.h>
-#include <linux/ethtool.h>
 #include <linux/netlink.h>
 #include <sys/mman.h>
 #include <sys/ioctl.h>
@@ -76,6 +75,8 @@
 
 extern int nl_sock;
 
+static pthread_mutex_t host_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 /*  Foward struct declarations */
 struct nic_ops qedi_op;
 
@@ -441,7 +442,7 @@ static int qedi_open(nic_t *nic)
 	qedi_t *bp = NULL;
 	struct stat uio_stat;
 	int i, rc;
-	int count;
+	size_t count;
 	uint32_t bus;
 	uint32_t slot;
 	uint32_t func;
@@ -665,7 +666,7 @@ static int qedi_open(nic_t *nic)
 		 nic->mac_addr[0], nic->mac_addr[1], nic->mac_addr[2],
 		 nic->mac_addr[3], nic->mac_addr[4], nic->mac_addr[5]);
 
-	qedi_get_library_name(&nic->library_name, (size_t *)&count);
+	qedi_get_library_name(&nic->library_name, &count);
 	LOG_INFO("%s: qedi initialized", nic->log_name);
 
 	bp->flags |= QEDI_OPENED;
@@ -814,15 +815,26 @@ static void qedi_prepare_xmit_packet(nic_t *nic,
 	struct uip_vlan_eth_hdr *eth_vlan = (struct uip_vlan_eth_hdr *)pkt->buf;
 	struct uip_eth_hdr *eth = (struct uip_eth_hdr *)bp->tx_pkt;
 
+	LOG_DEBUG(PFX "%s: pkt->buf_size=%d tpid=0x%x", nic->log_name,
+		  pkt->buf_size, eth_vlan->tpid);
+	
 	if (eth_vlan->tpid == htons(UIP_ETHTYPE_8021Q)) {
 		memcpy(bp->tx_pkt, pkt->buf, sizeof(struct uip_eth_hdr));
 		eth->type = eth_vlan->type;
 		pkt->buf_size -= (sizeof(struct uip_vlan_eth_hdr) -
 				  sizeof(struct uip_eth_hdr));
+
+	LOG_DEBUG(PFX "%s: pkt->buf_size=%d type=0x%x", nic->log_name,
+		  pkt->buf_size, eth->type);
+	LOG_DEBUG(PFX "%s: pkt->buf_size - eth_hdr_size = %d", nic->log_name,
+		  pkt->buf_size - sizeof(struct uip_eth_hdr));
+
 		memcpy(bp->tx_pkt + sizeof(struct uip_eth_hdr),
 		       pkt->buf + sizeof(struct uip_vlan_eth_hdr),
 		       pkt->buf_size - sizeof(struct uip_eth_hdr));
 	} else {
+		LOG_DEBUG(PFX "%s: NO VLAN pkt->buf_size=%d", nic->log_name,
+			  pkt->buf_size);
 		memcpy(bp->tx_pkt, pkt->buf, pkt->buf_size);
 	}
 
@@ -878,11 +890,18 @@ void qedi_start_xmit(nic_t *nic, size_t len, u16_t vlan_id)
 	path_data->handle = QEDI_PATH_HANDLE;
 	path_data->vlan_id = vlan_id;
 	uctrl->host_tx_pkt_len = len;
+	LOG_DEBUG(PFX "%s: host_no:%d vlan_id=%d, tx_pkt_len=%d",
+		  nic->log_name, ev->u.set_path.host_no, path_data->vlan_id, uctrl->host_tx_pkt_len);
 
+	LOG_DEBUG(PFX "%s: ACQUIRE HOST MUTEX", nic->log_name);
+	pthread_mutex_lock(&host_mutex);
 	rc = __kipc_call(nl_sock, ev, buflen);
 	if (rc > 0) {
 		bp->tx_prod++;
 		uctrl->host_tx_prod++;
+		LOG_DEBUG(PFX "%s: bp->tx_prod: %d, uctrl->host_tx_prod=%d",
+			  nic->log_name, bp->tx_prod, uctrl->host_tx_prod);
+
 		msync(uctrl, sizeof(struct qedi_uio_ctrl), MS_SYNC);
 		LOG_PACKET(PFX "%s: sent %d bytes using bp->tx_prod: %d",
 			   nic->log_name, len, bp->tx_prod);
@@ -890,6 +909,8 @@ void qedi_start_xmit(nic_t *nic, size_t len, u16_t vlan_id)
 		LOG_ERR(PFX "Pkt transmission failed: %d", rc);
 	}
 
+	LOG_DEBUG(PFX "%s: RELEASE HOST MUTEX", nic->log_name);
+	pthread_mutex_unlock(&host_mutex);
 	free(ubuf);
 }
 
@@ -925,14 +946,18 @@ int qedi_write(nic_t *nic, nic_interface_t *nic_iface, packet_t *pkt)
 		struct timespec sleep_req = {.tv_sec = 0, .tv_nsec = 5000000 },
 		    sleep_rem;
 
+		LOG_DEBUG(PFX "%s: host:%d - calling clear_tx_intr from qedi_write",
+			  nic->log_name, nic->host_no);
 		if (qedi_clear_tx_intr(nic) == 0)
 			break;
 
 		nanosleep(&sleep_req, &sleep_rem);
 	}
 
+	LOG_DEBUG(PFX "%s: host:%d - try getting xmit mutex",
+		   nic->log_name, nic->host_no);
 	if (pthread_mutex_trylock(&nic->xmit_mutex) != 0) {
-		LOG_PACKET(PFX "%s: Dropped previous transmitted packet",
+		LOG_DEBUG(PFX "%s: Dropped previous transmitted packet",
 			   nic->log_name);
 		return -EINVAL;
 	}
@@ -946,7 +971,7 @@ int qedi_write(nic_t *nic, nic_interface_t *nic_iface, packet_t *pkt)
 	nic->stats.tx.packets++;
 	nic->stats.tx.bytes += uip->uip_len;
 
-	LOG_PACKET(PFX "%s: transmitted %d bytes dev->tx_cons: %d, dev->tx_prod: %d, dev->tx_bd_prod:%d",
+	LOG_DEBUG(PFX "%s: transmitted %d bytes dev->tx_cons: %d, dev->tx_prod: %d, dev->tx_bd_prod:%d",
 		   nic->log_name, pkt->buf_size,
 		   bp->tx_cons, bp->tx_prod, bp->tx_bd_prod);
 
@@ -969,7 +994,7 @@ static int qedi_read(nic_t *nic, packet_t *pkt)
 	void *rx_pkt;
 	int rc = 0;
 	uint32_t sw_cons, bd_cons;
-	uint32_t hw_prod;
+	uint32_t hw_prod, bd_prod;
 	uint32_t rx_pkt_idx;
 	int len;
 	struct qedi_rx_bd *rx_bd;
@@ -988,14 +1013,20 @@ static int qedi_read(nic_t *nic, packet_t *pkt)
 	msync(bp->rx_comp_ring, nic->page_size, MS_SYNC);
 	uctrl = (struct qedi_uio_ctrl *)bp->uctrl_map;
 	hw_prod = uctrl->hw_rx_prod;
+	bd_prod = uctrl->hw_rx_bd_prod;
 	sw_cons = uctrl->host_rx_cons;
 	bd_cons = uctrl->host_rx_bd_cons;
-	rx_bd = bp->rx_comp_ring + (bd_cons * sizeof(*rx_bd));
+	rx_bd = bp->rx_comp_ring + (bd_prod * sizeof(*rx_bd));
 	len = rx_bd->rx_pkt_len;
 	rx_pkt_idx = rx_bd->rx_pkt_index;
 	vlan_id = rx_bd->vlan_id;
 
-	if (sw_cons != hw_prod) {
+	LOG_DEBUG(PFX "%s:hw_prod %d bd_prod %d, rx_pkt_idx %d, rxlen %d",
+		  nic->log_name, hw_prod, bd_prod, rx_bd->rx_pkt_index, len);
+	LOG_DEBUG(PFX "%s: sw_con %d bd_cons %d num BD %d",
+		  nic->log_name, sw_cons, bd_cons, QEDI_NUM_RX_BD);
+
+	if (bd_cons != bd_prod) {
 		LOG_DEBUG(PFX "%s: clearing rx interrupt: %d %d",
 			  nic->log_name, sw_cons, hw_prod);
 		rc = 1;
@@ -1073,7 +1104,7 @@ static int qedi_clear_tx_intr(nic_t *nic)
 		return -EINVAL;
 	}
 
-	LOG_PACKET(PFX "%s: clearing tx interrupt [%d %d]",
+	LOG_DEBUG(PFX "%s: clearing tx interrupt [%d %d]",
 		   nic->log_name, bp->tx_cons, hw_cons);
 	bp->tx_cons = hw_cons;
 
@@ -1085,7 +1116,7 @@ static int qedi_clear_tx_intr(nic_t *nic)
 		packet_t *pkt;
 		int i;
 
-		LOG_PACKET(PFX "%s: sending queued tx packet", nic->log_name);
+		LOG_DEBUG(PFX "%s: sending queued tx packet", nic->log_name);
 		pkt = nic_dequeue_tx_packet(nic);
 
 		/* Got a TX packet buffer of the TX queue and put it onto
@@ -1098,7 +1129,7 @@ static int qedi_clear_tx_intr(nic_t *nic)
 					(pkt->nic_iface->vlan_priority << 12) |
 					pkt->nic_iface->vlan_id);
 
-			LOG_PACKET(PFX "%s: transmitted queued packet %d bytes, dev->tx_cons: %d, dev->tx_prod: %d, dev->tx_bd_prod:%d",
+			LOG_DEBUG(PFX "%s: transmitted queued packet %d bytes, dev->tx_cons: %d, dev->tx_prod: %d, dev->tx_bd_prod:%d",
 				   nic->log_name, pkt->buf_size,
 				   bp->tx_cons, bp->tx_prod, bp->tx_bd_prod);
 
@@ -1126,6 +1157,8 @@ static int qedi_clear_tx_intr(nic_t *nic)
 		}
 	}
 
+	LOG_DEBUG(PFX "%s: host:%d - releasing xmit mutex",
+		   nic->log_name, nic->host_no);
 	pthread_mutex_unlock(&nic->xmit_mutex);
 
 	return 0;
