@@ -76,14 +76,12 @@ static const char uio_uevent_path_template[] = "/sys/class/uio/uio%d/uevent";
 
 static const char base_iscsi_host_name[] = "/sys/class/iscsi_host/";
 static const char host_template[] = "host%d";
-static const char iscsi_host_path_template[] = "/sys/class/iscsi_host/host%d";
 static const char iscsi_host_path_netdev_template[] =
 	"/sys/class/iscsi_host/host%d/netdev";
 static const char cnic_uio_sysfs_resc_template[] =
 	"/sys/class/uio/uio%i/device/resource%i";
 static const char iscsi_transport_handle_template[] =
 	"/sys/class/iscsi_transport/%s/handle";
-static const char eth_pfx[] = "eth";
 static const char host_pfx[] = "host";
 
 /**
@@ -247,6 +245,7 @@ int nic_discover_iscsi_hosts()
 						"space for NIC %s "
 						"during scan", raw);
 
+					free(raw);
 					rc = -ENOMEM;
 					break;
 				}
@@ -307,7 +306,7 @@ static int nic_util_enable_disable_multicast(nic_t *nic, uint32_t cmd)
 	/* Prepare the request */
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, nic->eth_device_name,
-		sizeof(nic->eth_device_name));
+		sizeof(ifr.ifr_name));
 	memcpy(ifr.ifr_hwaddr.sa_data, multicast_addr.addr, ETH_ALEN);
 
 	fd = socket(AF_INET, SOCK_DGRAM, 0);
@@ -1253,7 +1252,7 @@ int get_iscsi_transport_handle(nic_t *nic, uint64_t *handle)
 	if (rc != 0)
 		goto error;
 
-	elements_read = sscanf(raw, "%lu", handle);
+	elements_read = sscanf(raw, "%" PRIu64, handle);
 	if (elements_read != 1) {
 		LOG_ERR(PFX "%s: Couldn't parse transport handle from %s",
 			nic->log_name, temp_path);
@@ -1435,6 +1434,10 @@ nic_interface_t *nic_find_nic_iface(nic_t *nic,
 	nic_interface_t *current_vlan = NULL;
 
 	while (current != NULL) {
+		LOG_DEBUG(PFX "%s: incoming protocol: %d, vlan_id:%d iface_num: %d, request_type: %d",
+			  nic->log_name, protocol, vlan_id, iface_num,  request_type);
+		LOG_DEBUG(PFX "%s: host:%d iface_num: 0x%x VLAN: %d protocol: %d",
+			  nic->log_name, nic->host_no, current->iface_num, current->vlan_id, current->protocol);
 		if (current->protocol != protocol)
 			goto next;
 
