@@ -27,8 +27,10 @@
 #include <errno.h>
 #include <dirent.h>
 #include <limits.h>
+#include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/file.h>
+#include <inttypes.h>
 
 #include "idbm.h"
 #include "idbm_fields.h"
@@ -48,6 +50,8 @@
 
 static struct idbm *db;
 
+#define ARRAY_LEN(x) ( sizeof(x) / sizeof((x)[0]) )
+
 #define __recinfo_str(_key, _info, _rec, _name, _show, _n, _mod) do { \
 	_info[_n].type = TYPE_STR; \
 	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
@@ -64,7 +68,7 @@ static struct idbm *db;
 #define __recinfo_int(_key, _info, _rec, _name, _show, _n, _mod) do { \
 	_info[_n].type = TYPE_INT; \
 	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
-	snprintf(_info[_n].value, VALUE_MAXVAL, "%d", _rec->_name); \
+	snprintf(_info[_n].value, VALUE_MAXVAL, "%" PRIi32, _rec->_name); \
 	_info[_n].data = &_rec->_name; \
 	_info[_n].data_len = sizeof(_rec->_name); \
 	_info[_n].visible = _show; \
@@ -75,7 +79,7 @@ static struct idbm *db;
 #define __recinfo_uint8(_key, _info, _rec, _name, _show, _n, _mod) do { \
 	_info[_n].type = TYPE_UINT8; \
 	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
-	snprintf(_info[_n].value, VALUE_MAXVAL, "%d", _rec->_name); \
+	snprintf(_info[_n].value, VALUE_MAXVAL, "%" PRIu8, _rec->_name); \
 	_info[_n].data = &_rec->_name; \
 	_info[_n].data_len = sizeof(_rec->_name); \
 	_info[_n].visible = _show; \
@@ -86,7 +90,7 @@ static struct idbm *db;
 #define __recinfo_uint16(_key, _info, _rec, _name, _show, _n, _mod) do { \
 	_info[_n].type = TYPE_UINT16; \
 	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
-	snprintf(_info[_n].value, VALUE_MAXVAL, "%d", _rec->_name); \
+	snprintf(_info[_n].value, VALUE_MAXVAL, "%" PRIu16, _rec->_name); \
 	_info[_n].data = &_rec->_name; \
 	_info[_n].data_len = sizeof(_rec->_name); \
 	_info[_n].visible = _show; \
@@ -97,7 +101,7 @@ static struct idbm *db;
 #define __recinfo_uint32(_key, _info, _rec, _name, _show, _n, _mod) do { \
 	_info[_n].type = TYPE_UINT32; \
 	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
-	snprintf(_info[_n].value, VALUE_MAXVAL, "%d", _rec->_name); \
+	snprintf(_info[_n].value, VALUE_MAXVAL, "%" PRIu32, _rec->_name); \
 	_info[_n].data = &_rec->_name; \
 	_info[_n].data_len = sizeof(_rec->_name); \
 	_info[_n].visible = _show; \
@@ -162,6 +166,42 @@ static struct idbm *db;
 	_n++; \
 } while(0)
 
+#define __recinfo_int_list(_key,_info,_rec,_name,_show,_tbl,_n,_mod) do { \
+	_info[_n].type = TYPE_INT_LIST; \
+	strlcpy(_info[_n].name, _key, NAME_MAXVAL); \
+	for (unsigned long _i = 0; _i < ARRAY_LEN(_rec->_name); _i++) {	\
+		if (_rec->_name[_i] != (unsigned)~0) {			\
+			for (unsigned long _j = 0; _j < ARRAY_LEN(_tbl); _j++) {	\
+				if (_tbl[_j].value == (int)_rec->_name[_i]) { \
+					strcat(_info[_n].value, _tbl[_j].name); \
+					strcat(_info[_n].value, ","); \
+					break; \
+				} \
+			} \
+		} \
+	} \
+	/* delete trailing ',' */ \
+	if (strrchr(_info[_n].value, ',')) \
+		*strrchr(_info[_n].value, ',') = '\0'; \
+	_info[_n].data = &_rec->_name; \
+	_info[_n].data_len = sizeof(_rec->_name); \
+	_info[_n].visible = _show; \
+	_info[_n].opts[0] = (void *)&_tbl; \
+	_info[_n].numopts = ARRAY_LEN(_tbl); \
+	_info[_n].can_modify = _mod; \
+	_n++; \
+} while (0)
+
+static struct int_list_tbl {
+	const char *name;
+	int value;
+} chap_algs [] = {
+	{ "MD5", AUTH_CHAP_ALG_MD5 },
+	{ "SHA1", AUTH_CHAP_ALG_SHA1 },
+	{ "SHA256", AUTH_CHAP_ALG_SHA256 },
+	{ "SHA3-256", AUTH_CHAP_ALG_SHA3_256 },
+};
+
 static void
 idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 {
@@ -194,6 +234,10 @@ idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 		__recinfo_int(DISC_ST_PASSWORD_IN_LEN, ri, r,
 			u.sendtargets.auth.password_in_length, IDBM_HIDE,
 			num, 1);
+		/* reusing SESSION_CHAP_ALGS */
+		__recinfo_int_list(SESSION_CHAP_ALGS, ri, r,
+				   u.sendtargets.auth.chap_algs,
+				   IDBM_SHOW, chap_algs, num, 1);
 		__recinfo_int(DISC_ST_LOGIN_TMO, ri, r,
 			u.sendtargets.conn_timeo.login_timeout,
 			IDBM_SHOW, num, 1);
@@ -426,6 +470,8 @@ idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 		      session.auth.password_in, IDBM_MASKED, num, 1);
 	__recinfo_int(SESSION_PASSWORD_IN_LEN, ri, r,
 		      session.auth.password_in_length, IDBM_HIDE, num, 1);
+	__recinfo_int_list(SESSION_CHAP_ALGS, ri, r,
+			   session.auth.chap_algs, IDBM_SHOW, chap_algs, num, 1);
 	__recinfo_int(SESSION_REPLACEMENT_TMO, ri, r,
 		      session.timeo.replacement_timeout,
 		      IDBM_SHOW, num, 1);
@@ -467,6 +513,8 @@ idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 	__recinfo_int_o2(SESSION_SCAN, ri, r,
 			 session.scan, IDBM_SHOW, "manual", "auto",
 			 num, 1);
+	__recinfo_int(SESSION_REOPEN_MAX, ri, r,
+			session.reopen_max, IDBM_SHOW, num, 1);
 
 	for (i = 0; i < ISCSI_CONN_MAX; i++) {
 		char key[NAME_MAXVAL];
@@ -929,6 +977,9 @@ idbm_discovery_setup_defaults(discovery_rec_t *rec, discovery_type_e type)
 		rec->u.sendtargets.auth.authmethod = 0;
 		rec->u.sendtargets.auth.password_length = 0;
 		rec->u.sendtargets.auth.password_in_length = 0;
+		/* TYPE_INT_LIST fields should be initialized to ~0 to indicate unset values */
+		memset(rec->u.sendtargets.auth.chap_algs, ~0, sizeof(rec->u.sendtargets.auth.chap_algs));
+		rec->u.sendtargets.auth.chap_algs[0] = AUTH_CHAP_ALG_MD5;
 		rec->u.sendtargets.conn_timeo.login_timeout=15;
 		rec->u.sendtargets.conn_timeo.auth_timeout = 45;
 		rec->u.sendtargets.conn_timeo.active_timeout=30;
@@ -962,59 +1013,111 @@ int idbm_rec_update_param(recinfo_t *info, char *name, char *value,
 	int i;
 	int passwd_done = 0;
 	char passwd_len[8];
+	char *tmp_value, *token;
+	bool *found;
+	int *tmp_data;
 
 setup_passwd_len:
 	for (i=0; i<MAX_KEYS; i++) {
 		if (!strcmp(name, info[i].name)) {
-			int j;
+			int j,k;
+			struct int_list_tbl *tbl;
 
 			log_debug(7, "updated '%s', '%s' => '%s'", name,
 				  info[i].value, value);
 			/* parse recinfo by type */
-			if (info[i].type == TYPE_INT) {
+			switch (info[i].type) {
+			case TYPE_INT:
 				if (!info[i].data)
 					continue;
 
 				*(int*)info[i].data =
 					strtoul(value, NULL, 10);
 				goto updated;
-			} else if (info[i].type == TYPE_UINT8) {
+			case TYPE_UINT8:
 				if (!info[i].data)
 					continue;
 
 				*(uint8_t *)info[i].data =
 					strtoul(value, NULL, 10);
 				goto updated;
-			} else if (info[i].type == TYPE_UINT16) {
+			case TYPE_UINT16:
 				if (!info[i].data)
 					continue;
 
 				*(uint16_t *)info[i].data =
 					strtoul(value, NULL, 10);
 				goto updated;
-			} else if (info[i].type == TYPE_UINT32) {
+			case TYPE_UINT32:
 				if (!info[i].data)
 					continue;
 
 				*(uint32_t *)info[i].data =
 					strtoul(value, NULL, 10);
 				goto updated;
-			} else if (info[i].type == TYPE_STR) {
+			case TYPE_STR:
 				if (!info[i].data)
 					continue;
 
 				strlcpy((char*)info[i].data,
 					value, info[i].data_len);
 				goto updated;
-			}
-			for (j=0; j<info[i].numopts; j++) {
-				if (!strcmp(value, info[i].opts[j])) {
-					if (!info[i].data)
-						continue;
+			case TYPE_INT_O:
+				for (j=0; j<info[i].numopts; j++) {
+					if (!strcmp(value, info[i].opts[j])) {
+						if (!info[i].data)
+							continue;
 
-					*(int*)info[i].data = j;
-					goto updated;
+						*(int*)info[i].data = j;
+						goto updated;
+					}
 				}
+				/* internal error if reached ?? */
+				break;
+			case TYPE_INT_LIST:
+				if (!info[i].data)
+					continue;
+				tbl = (void *)info[i].opts[0];
+				/* strsep is destructive, make a copy to work with */
+				tmp_value = strdup(value);
+				k = 0;
+				tmp_data = malloc(info[i].data_len);
+				memset(tmp_data, ~0, info[i].data_len);
+				found = calloc(info[i].numopts, sizeof(bool));
+
+next_token:			while ((token = strsep(&tmp_value, ", \n"))) {
+					if (!strlen(token))
+						continue;
+					if ((k * (int)sizeof(int)) >= (info[i].data_len)) {
+						log_warning("Too many values set for '%s'"
+						            ", continuing without processing them all",
+						            info[i].name);
+						break;
+					}
+					for (j = 0; j < info[i].numopts; j++) {
+						if (!strcmp(token, tbl[j].name)) {
+							if ((found[j])) {
+								log_warning("Ignoring repeated "
+								            "value '%s' "
+								            "for '%s'", token,
+								            info[i].name);
+								goto next_token;
+							}
+							((int*)tmp_data)[k++] = tbl[j].value;
+							found[j] = true;
+							goto next_token;
+						}
+					}
+					log_warning("Ignoring unknown value '%s'"
+					            " for '%s'", token, info[i].name);
+				}
+				memcpy(info[i].data, tmp_data, info[i].data_len);
+				free(tmp_value);
+				free(tmp_data);
+				tmp_value = NULL;
+				tmp_data = NULL;
+				token = NULL;
+				goto updated;
 			}
 			if (line_number) {
 				log_warning("config file line %d contains "
@@ -1038,7 +1141,7 @@ updated:
 	if (!passwd_done && !strcmp(#_param, name)) { \
 		passwd_done = 1; \
 		name = #_param "_length"; \
-		snprintf(passwd_len, 8, "%d", (int)strlen(value)); \
+		snprintf(passwd_len, 8, "%.7" PRIi32, (int)strlen(value) & 0xffff); \
 		value = passwd_len; \
 		goto setup_passwd_len; \
 	}
@@ -1237,14 +1340,6 @@ int idbm_print_node_info(void *data, node_rec_t *rec)
 	return 0;
 }
 
-int idbm_print_iface_info(void *data, struct iface_rec *iface)
-{
-	int show = *((int *)data);
-
-	idbm_print(IDBM_PRINT_TYPE_IFACE, iface, show, stdout);
-	return 0;
-}
-
 int idbm_print_host_chap_info(struct iscsi_chap_rec *chap)
 {
 	/* User only calls this to print chap so always print */
@@ -1258,7 +1353,7 @@ int idbm_print_flashnode_info(struct flashnode_rec *fnode)
 	return 0;
 }
 
-int idbm_print_node_flat(void *data, node_rec_t *rec)
+int idbm_print_node_flat(__attribute__((unused))void *data, node_rec_t *rec)
 {
 	if (strchr(rec->conn[0].address, '.'))
 		printf("%s:%d,%d %s\n", rec->conn[0].address, rec->conn[0].port,
@@ -1402,7 +1497,12 @@ static FILE *idbm_open_rec_r(char *portal, char *config)
 	return fopen(portal, "r");
 }
 
-static int __idbm_rec_read(node_rec_t *out_rec, char *conf)
+/*
+ * When the disable_lock param is true, the idbm_lock/idbm_unlock needs
+ * to be holt by the caller, this will avoid overwriting each other in
+ * case of updating(read-modify-write) the recs in parallel.
+ */
+static int __idbm_rec_read(node_rec_t *out_rec, char *conf, bool disable_lock)
 {
 	recinfo_t *info;
 	FILE *f;
@@ -1412,9 +1512,11 @@ static int __idbm_rec_read(node_rec_t *out_rec, char *conf)
 	if (!info)
 		return ISCSI_ERR_NOMEM;
 
-	rc = idbm_lock();
-	if (rc)
-		goto free_info;
+	if (!disable_lock) {
+		rc = idbm_lock();
+		if (rc)
+			goto free_info;
+	}
 
 	f = fopen(conf, "r");
 	if (!f) {
@@ -1431,15 +1533,21 @@ static int __idbm_rec_read(node_rec_t *out_rec, char *conf)
 	fclose(f);
 
 unlock:
-	idbm_unlock();
+	if (!disable_lock)
+		idbm_unlock();
 free_info:
 	free(info);
 	return rc;
 }
 
+/*
+ * When the disable_lock param is true, the idbm_lock/idbm_unlock needs
+ * to be holt by the caller, this will avoid overwriting each other in
+ * case of updating(read-modify-write) the recs in parallel.
+ */
 int
 idbm_rec_read(node_rec_t *out_rec, char *targetname, int tpgt,
-	      char *ip, int port, struct iface_rec *iface)
+	      char *ip, int port, struct iface_rec *iface, bool disable_lock)
 {
 	struct stat statb;
 	char *portal;
@@ -1471,7 +1579,7 @@ idbm_rec_read(node_rec_t *out_rec, char *targetname, int tpgt,
 	}
 
 read:
-	rc = __idbm_rec_read(out_rec, portal);
+	rc = __idbm_rec_read(out_rec, portal, disable_lock);
 free_portal:
 	free(portal);
 	return rc;
@@ -1527,7 +1635,7 @@ static int idbm_print_discovered(discovery_rec_t *drec, int info_level)
 	int num_found = 0;
 
 	if (info_level < 1)
-		idbm_for_each_rec(&num_found, drec, print_discovered_flat);
+		idbm_for_each_rec(&num_found, drec, print_discovered_flat, false);
 	else {
 		struct discovered_tree_info tree_info;
 		struct node_rec last_rec;
@@ -1537,7 +1645,7 @@ static int idbm_print_discovered(discovery_rec_t *drec, int info_level)
 		tree_info.drec = drec;
 		tree_info.last_rec = &last_rec;
 
-		idbm_for_each_rec(&num_found, &tree_info,							  print_discovered_tree);
+		idbm_for_each_rec(&num_found, &tree_info, print_discovered_tree, false);
 	}
 	return num_found;
 }
@@ -1709,9 +1817,9 @@ int idbm_print_all_discovery(int info_level)
  * fn should return -1 if it skipped the rec, an ISCSI_ERR error code if
  * the operation failed or 0 if fn was run successfully.
  */
-static int idbm_for_each_iface(int *found, void *data,
-				idbm_iface_op_fn *fn,
-				char *targetname, int tpgt, char *ip, int port)
+static int idbm_for_each_iface(int *found, void *data, idbm_iface_op_fn *fn,
+			       char *targetname, int tpgt, char *ip, int port,
+			       bool ruw_lock)
 {
 	DIR *iface_dirfd;
 	struct dirent *iface_dent;
@@ -1736,7 +1844,7 @@ static int idbm_for_each_iface(int *found, void *data,
 		goto free_portal;
 	}
 
-	rc = __idbm_rec_read(&rec, portal);
+	rc = __idbm_rec_read(&rec, portal, ruw_lock);
 	if (rc)
 		goto free_portal;
 
@@ -1769,7 +1877,7 @@ read_iface:
 		memset(portal, 0, PATH_MAX);
 		snprintf(portal, PATH_MAX, "%s/%s/%s,%d,%d/%s", NODE_CONFIG_DIR,
 			 targetname, ip, port, tpgt, iface_dent->d_name);
-		if (__idbm_rec_read(&rec, portal))
+		if (__idbm_rec_read(&rec, portal, ruw_lock))
 			continue;
 
 		curr_rc = fn(data, &rec);
@@ -1791,7 +1899,7 @@ free_portal:
  * The portal could be a file or dir with interfaces
  */
 int idbm_for_each_portal(int *found, void *data, idbm_portal_op_fn *fn,
-			 char *targetname)
+			 char *targetname, bool ruw_lock)
 {
 	DIR *portal_dirfd;
 	struct dirent *portal_dent;
@@ -1828,7 +1936,8 @@ int idbm_for_each_portal(int *found, void *data, idbm_portal_op_fn *fn,
 
 		curr_rc = fn(found, data, targetname,
 			tmp_tpgt ? atoi(tmp_tpgt) : -1,
-			portal_dent->d_name, atoi(tmp_port));
+			portal_dent->d_name, atoi(tmp_port),
+			ruw_lock);
 		/* less than zero means it was not a match */
 		if (curr_rc > 0 && !rc)
 			rc = curr_rc;
@@ -1839,7 +1948,7 @@ done:
 	return rc;
 }
 
-int idbm_for_each_node(int *found, void *data, idbm_node_op_fn *fn)
+int idbm_for_each_node(int *found, void *data, idbm_node_op_fn *fn, bool ruw_lock)
 {
 	DIR *node_dirfd;
 	struct dirent *node_dent;
@@ -1860,7 +1969,7 @@ int idbm_for_each_node(int *found, void *data, idbm_node_op_fn *fn)
 			continue;
 
 		log_debug(5, "searching %s", node_dent->d_name);
-		curr_rc = fn(found, data, node_dent->d_name);
+		curr_rc = fn(found, data, node_dent->d_name, ruw_lock);
 		/* less than zero means it was not a match */
 		if (curr_rc > 0 && !rc)
 			rc = curr_rc;
@@ -1878,18 +1987,30 @@ static int iface_fn(void *data, node_rec_t *rec)
 }
 
 static int portal_fn(int *found, void *data, char *targetname,
-		     int tpgt, char *ip, int port)
+		     int tpgt, char *ip, int port, bool ruw_lock)
 {
-	return idbm_for_each_iface(found, data, iface_fn, targetname,
-				   tpgt, ip, port);
+	int rc;
+
+	if (ruw_lock) {
+		rc = idbm_lock();
+		if (rc)
+			return rc;
+	}
+
+	rc = idbm_for_each_iface(found, data, iface_fn, targetname,
+			         tpgt, ip, port, ruw_lock);
+	if (ruw_lock)
+		idbm_unlock();
+
+	return rc;
 }
 
-static int node_fn(int *found, void *data, char *targetname)
+static int node_fn(int *found, void *data, char *targetname, bool ruw_lock)
 {
-	return idbm_for_each_portal(found, data, portal_fn, targetname);
+	return idbm_for_each_portal(found, data, portal_fn, targetname, ruw_lock);
 }
 
-int idbm_for_each_rec(int *found, void *data, idbm_iface_op_fn *fn)
+int idbm_for_each_rec(int *found, void *data, idbm_iface_op_fn *fn, bool ruw_lock)
 {
 	struct rec_op_data op_data;
 
@@ -1897,7 +2018,7 @@ int idbm_for_each_rec(int *found, void *data, idbm_iface_op_fn *fn)
 	op_data.data = data;
 	op_data.fn = fn;
 
-	return idbm_for_each_node(found, &op_data, node_fn);
+	return idbm_for_each_node(found, &op_data, node_fn, ruw_lock);
 }
 
 static struct {
@@ -1956,7 +2077,7 @@ idbm_discovery_read(discovery_rec_t *out_rec, int drec_type,
 	idbm_recinfo_config(info, f);
 	fclose(f);
 
-unlock:	
+unlock:
 	idbm_unlock();
 free_info:
 	free(portal);
@@ -2008,7 +2129,12 @@ mkdir_portal:
 	return f;
 }
 
-static int idbm_rec_write(node_rec_t *rec)
+/*
+ * When the disable_lock param is true, the idbm_lock/idbm_unlock needs
+ * to be holt by the caller, this will avoid overwriting each other in
+ * case of updating(read-modify-write) the recs in parallel.
+ */
+static int idbm_rec_write(node_rec_t *rec, bool disable_lock)
 {
 	struct stat statb;
 	FILE *f;
@@ -2045,9 +2171,11 @@ static int idbm_rec_write(node_rec_t *rec)
 		 rec->name, rec->conn[0].address, rec->conn[0].port);
 	log_debug(5, "Looking for config file %s", portal);
 
-	rc = idbm_lock();
-	if (rc)
-		goto free_portal;
+	if (!disable_lock) {
+		rc = idbm_lock();
+		if (rc)
+			goto free_portal;
+	}
 
 	rc = stat(portal, &statb);
 	if (rc) {
@@ -2085,7 +2213,7 @@ static int idbm_rec_write(node_rec_t *rec)
 	} else {
 		rc = ISCSI_ERR_INVAL;
 		goto unlock;
-	}	
+	}
 
 mkdir_portal:
 	snprintf(portal, PATH_MAX, "%s/%s/%s,%d,%d", NODE_CONFIG_DIR,
@@ -2113,7 +2241,8 @@ open_conf:
 	idbm_print(IDBM_PRINT_TYPE_NODE, rec, 1, f);
 	fclose(f);
 unlock:
-	idbm_unlock();
+	if (!disable_lock)
+		idbm_unlock();
 free_portal:
 	free(portal);
 	return rc;
@@ -2288,18 +2417,24 @@ static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 int idbm_add_node(node_rec_t *newrec, discovery_rec_t *drec, int overwrite)
 {
 	node_rec_t rec;
-	char *node_portal, *disc_portal;
+	char *node_portal = NULL, *disc_portal;
 	int rc;
 
+	rc = idbm_lock();
+	if (rc)
+		return rc;
+
 	if (!idbm_rec_read(&rec, newrec->name, newrec->tpgt,
 			   newrec->conn[0].address, newrec->conn[0].port,
-			   &newrec->iface)) {
-		if (!overwrite)
-			return 0;
+			   &newrec->iface, true)) {
+		if (!overwrite) {
+			rc = 0;
+			goto unlock;
+		}
 
 		rc = idbm_delete_node(&rec);
 		if (rc)
-			return rc;
+			goto unlock;
 		log_debug(7, "overwriting existing record");
 	} else
 		log_debug(7, "adding new DB record");
@@ -2310,17 +2445,19 @@ int idbm_add_node(node_rec_t *newrec, discovery_rec_t *drec, int overwrite)
 		strcpy(newrec->disc_address, drec->address);
 	}
 
-	rc = idbm_rec_write(newrec);
+	rc = idbm_rec_write(newrec, true);
 	/*
 	 * if a old app passed in a bogus tpgt then we do not create links
 	 * since it will set a different tpgt in another iscsiadm call
 	 */
 	if (rc || !drec || newrec->tpgt == PORTAL_GROUP_TAG_UNKNOWN)
-		return rc;
+		goto unlock;
 
 	node_portal = calloc(2, PATH_MAX);
-	if (!node_portal)
-		return ISCSI_ERR_NOMEM;
+	if (!node_portal) {
+		rc = ISCSI_ERR_NOMEM;
+		goto unlock;
+	}
 
 	disc_portal = node_portal + PATH_MAX;
 	snprintf(node_portal, PATH_MAX, "%s/%s/%s,%d,%d", NODE_CONFIG_DIR,
@@ -2328,15 +2465,11 @@ int idbm_add_node(node_rec_t *newrec, discovery_rec_t *drec, int overwrite)
 		 newrec->tpgt);
 	rc = setup_disc_to_node_link(disc_portal, newrec);
 	if (rc)
-		goto free_portal;
+		goto unlock;
 
 	log_debug(7, "node addition making link from %s to %s", node_portal,
 		 disc_portal);
 
-	rc = idbm_lock();
-	if (rc)
-		goto free_portal;
-
 	if (symlink(node_portal, disc_portal)) {
 		if (errno == EEXIST)
 			log_debug(7, "link from %s to %s exists", node_portal,
@@ -2348,8 +2481,8 @@ int idbm_add_node(node_rec_t *newrec, discovery_rec_t *drec, int overwrite)
 				 strerror(errno));
 		}
 	}
+unlock:
 	idbm_unlock();
-free_portal:
 	free(node_portal);
 	return rc;
 }
@@ -2453,7 +2586,7 @@ int idbm_bind_ifaces_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
 	}
 	return 0;
 
-fail:	
+fail:
 	list_for_each_entry_safe(iface, tmp_iface, &def_ifaces, list) {
 		list_del(&iface->list);
 		free(iface);
@@ -2588,7 +2721,7 @@ static int idbm_remove_disc_to_node_link(node_rec_t *rec,
 		 rec->name, rec->conn[0].address, rec->conn[0].port, rec->tpgt,
 		 rec->iface.name);
 
-	rc = __idbm_rec_read(tmprec, portal);
+	rc = __idbm_rec_read(tmprec, portal, false);
 	if (rc) {
 		/* old style recs will not have tpgt or a link so skip */
 		rc = 0;
@@ -2815,7 +2948,7 @@ int idbm_node_set_param(void *data, node_rec_t *rec)
 	if (rc)
 		return rc;
 
-	return idbm_rec_write(rec);
+	return idbm_rec_write(rec, true);
 }
 
 int idbm_discovery_set_param(void *data, discovery_rec_t *rec)
@@ -2985,8 +3118,11 @@ void idbm_node_setup_defaults(node_rec_t *rec)
 	rec->session.queue_depth = QUEUE_DEPTH;
 	rec->session.nr_sessions = 1;
 	rec->session.initial_login_retry_max = DEF_INITIAL_LOGIN_RETRIES_MAX;
-	rec->session.reopen_max = 32;
+	rec->session.reopen_max = DEF_SESSION_REOPEN_MAX;
 	rec->session.auth.authmethod = 0;
+	/* TYPE_INT_LIST fields should be initialized to ~0 to indicate unset values */
+	memset(rec->session.auth.chap_algs, ~0, sizeof(rec->session.auth.chap_algs));
+	rec->session.auth.chap_algs[0] = AUTH_CHAP_ALG_MD5;
 	rec->session.auth.password_length = 0;
 	rec->session.auth.password_in_length = 0;
 	rec->session.err_timeo.abort_timeout = DEF_ABORT_TIMEO;
