@@ -44,6 +44,8 @@
 #include "iscsi_err.h"
 #include "iscsi_netlink.h"
 
+#define _unwrap(x) (x && strlen(x)) ? x : UNKNOWN_VALUE
+
 /*
  * Default ifaces for use with transports that do not bind to hardware
  * by defaults (transports that let the interconnect layer to the routing
@@ -511,7 +513,8 @@ static int iface_setup_binding_from_kern_iface(void *data,
 	return 0;
 }
 
-static int __iface_setup_host_bindings(void *data, struct host_info *hinfo)
+static int __iface_setup_host_bindings(__attribute__((unused))void *data,
+				       struct host_info *hinfo)
 {
 	struct iface_rec *def_iface;
 	struct iscsi_transport *t;
@@ -839,7 +842,8 @@ static int iface_print_nodes(void *data, node_rec_t *rec)
  * have the binding info. When we store the iface specific node settings
  * in the iface record then it will look different.
  */
-int iface_print_tree(void *data, struct iface_rec *iface)
+int iface_print_tree(__attribute__((unused))void *data,
+		     struct iface_rec *iface)
 {
 	struct node_rec last_rec;
 	struct iface_print_node_data print_data;
@@ -852,21 +856,19 @@ int iface_print_tree(void *data, struct iface_rec *iface)
 	print_data.match_iface = iface;
 	print_data.last_rec = &last_rec;
 
-	idbm_for_each_rec(&num_found, &print_data, iface_print_nodes);
+	idbm_for_each_rec(&num_found, &print_data, iface_print_nodes, false);
 	return 0;
 }
 
-int iface_print_flat(void *data, struct iface_rec *iface)
+void iface_print_flat(struct iscsi_iface *iface)
 {
 	printf("%s %s,%s,%s,%s,%s\n",
-		strlen(iface->name) ? iface->name : UNKNOWN_VALUE,
-		strlen(iface->transport_name) ? iface->transport_name :
-							UNKNOWN_VALUE,
-		strlen(iface->hwaddress) ? iface->hwaddress : UNKNOWN_VALUE,
-		strlen(iface->ipaddress) ? iface->ipaddress : UNKNOWN_VALUE,
-		strlen(iface->netdev) ? iface->netdev : UNKNOWN_VALUE,
-		strlen(iface->iname) ? iface->iname : UNKNOWN_VALUE);
-	return 0;
+	       _unwrap(iscsi_iface_name_get(iface)),
+	       _unwrap(iscsi_iface_transport_name_get(iface)),
+	       _unwrap(iscsi_iface_hwaddress_get(iface)),
+	       _unwrap(iscsi_iface_ipaddress_get(iface)),
+	       _unwrap(iscsi_iface_netdev_get(iface)),
+	       _unwrap(iscsi_iface_iname_get(iface)));
 }
 
 int iface_for_each_iface(void *data, int skip_def, int *nr_found,
