@@ -253,6 +253,7 @@ __session_conn_create(iscsi_session_t *session, int cid)
 {
 	iscsi_conn_t *conn = &session->conn[cid];
 	conn_rec_t *conn_rec = &session->nrec.conn[cid];
+	node_rec_t *rec = &session->nrec;
 	int err;
 
 	if (iscsi_ev_context_alloc(conn)) {
@@ -260,6 +261,9 @@ __session_conn_create(iscsi_session_t *session, int cid)
 		return ISCSI_ERR_NOMEM;
 	}
 
+	/* set session reconnection retry max */
+	session->reopen_max = rec->session.reopen_max;
+
 	conn->state = ISCSI_CONN_STATE_FREE;
 	conn->session = session;
 	actor_init(&conn->login_timer, iscsi_login_timedout, NULL);
@@ -352,7 +356,7 @@ __session_create(node_rec_t *rec, struct iscsi_transport *t, int *rc)
 	INIT_LIST_HEAD(&session->list);
 	session->t = t;
 	session->reopen_qtask.mgmt_ipc_fd = -1;
-	session->id = -1;
+	session->id = INVALID_SESSION_ID;
 	session->use_ipc = 1;
 
 	/* save node record. we might need it for redirection */
@@ -416,6 +420,9 @@ __session_create(node_rec_t *rec, struct iscsi_transport *t, int *rc)
                 }
         }
 
+	/* reset session reopen count */
+	session->reopen_cnt = 0;
+
 	list_add_tail(&session->list, &t->sessions);
 	return session;
 
@@ -469,7 +476,7 @@ session_conn_shutdown(iscsi_conn_t *conn, queue_task_t *qtask,
 	if (session->t->template->ep_disconnect)
 		session->t->template->ep_disconnect(conn);
 
-	if (session->id == -1)
+	if (session->id == INVALID_SESSION_ID)
 		goto cleanup;
 
 	if (!iscsi_sysfs_session_has_leadconn(session->id))
@@ -495,7 +502,7 @@ session_conn_shutdown(iscsi_conn_t *conn, queue_task_t *qtask,
 	}
 
 cleanup:
-	if (session->id != -1) {
+	if (session->id != INVALID_SESSION_ID) {
 		log_debug(2, "kdestroy session %u", session->id);
 		session->r_stage = R_STAGE_SESSION_DESTOYED;
 		if (ipc->destroy_session(session->t->handle, session->id)) {
@@ -724,8 +731,15 @@ static void iscsi_login_eh(struct iscsi_conn *conn, struct queue_task *qtask,
 			break;
 		case R_STAGE_SESSION_REOPEN:
 			log_debug(6, "login failed ISCSI_CONN_STATE_XPT_WAIT/"
-				  "R_STAGE_SESSION_REOPEN %d",
-				  session->reopen_cnt);
+				  "R_STAGE_SESSION_REOPEN (reopen_cnt=%d, reopen_max=%d)",
+				  session->reopen_cnt, session->reopen_max);
+			if (session->reopen_max &&
+			    (session->reopen_cnt > session->reopen_max)) {
+				log_info("Giving up on session %d after %d retries", 
+						session->id, session->reopen_max);
+				session_conn_shutdown(conn, qtask, err);
+				break;
+			}
 			/* timeout during reopen connect. try again */
 			session_conn_reopen(conn, qtask, 0);
 			break;
@@ -1544,7 +1558,7 @@ static void session_conn_poll(void *data)
 		memset(c, 0, sizeof(iscsi_login_context_t));
 
 		/* do not allocate new connection in case of reopen */
-		if (session->id == -1) {
+		if (session->id == INVALID_SESSION_ID) {
 			if (conn->id == 0 && session_ipc_create(session)) {
 				log_error("Can't create session.");
 				err = ISCSI_ERR_INTERNAL;
@@ -2056,8 +2070,22 @@ int session_logout_task(int sid, queue_task_t *qtask)
 		return ISCSI_ERR_SESS_NOT_FOUND;
 	}
 	conn = &session->conn[0];
+
 	/*
-	 * If syncing up or if this is the initial login and mgmt_ipc
+	 * If syncing up, in XPT_WAIT, and REOPENing, then return
+	 * an informative error, since the target for this session
+	 * is likely not connected
+	 */
+	if (session->notify_qtask &&
+	    (conn->state == ISCSI_CONN_STATE_XPT_WAIT) &&
+	    (session->r_stage == R_STAGE_SESSION_REOPEN)) {
+		log_warning("session cannot be terminted because it's trying to reconnect: try again later");
+		return ISCSI_ERR_SESSION_NOT_CONNECTED;
+	}
+
+	/*
+	 * If syncing up and not reconnecting,
+	 * or if this is the initial login and mgmt_ipc
 	 * has not been notified of that result fail the logout request
 	 */
 	if (session->notify_qtask ||
@@ -2109,7 +2137,9 @@ invalid_state:
 }
 
 int
-iscsi_host_send_targets(queue_task_t *qtask, int host_no, int do_login,
+iscsi_host_send_targets(__attribute__((unused))queue_task_t *qtask,
+			int host_no,
+			__attribute__((unused))int do_login,
 			struct sockaddr_storage *ss)
 {
 	struct iscsi_transport *t;
