@@ -33,6 +33,7 @@
 
 char *log_name;
 int log_level = 0;
+struct logarea *la = NULL;
 
 static int log_stop_daemon = 0;
 static void (*log_func)(int prio, void *priv, const char *fmt, va_list ap);
@@ -186,14 +187,17 @@ int log_enqueue (int prio, const char * fmt, va_list ap)
 
 	/* not enough space on tail : rewind */
 	if (la->head <= la->tail &&
-	    (len + sizeof(struct logmsg)) > (la->end - la->tail)) {
+	    (long)(len + sizeof(struct logmsg)) > (la->end - la->tail)) {
 		logdbg(stderr, "enqueue: rewind tail to %p\n", la->tail);
 			la->tail = la->start;
+
+			if (la->empty)
+				la->head = lastmsg = la->tail;
 	}
 
 	/* not enough space on head : drop msg */
 	if (la->head > la->tail &&
-	    (len + sizeof(struct logmsg)) > (la->head - la->tail)) {
+	    (long)(len + sizeof(struct logmsg)) > (la->head - la->tail)) {
 		logdbg(stderr, "enqueue: log area overrun, drop msg\n");
 
 		if (!la->empty)
@@ -259,7 +263,10 @@ static void log_syslog (void * buff)
 	syslog(msg->prio, "%s", (char *)&msg->str);
 }
 
-void log_do_log_daemon(int prio, void *priv, const char *fmt, va_list ap)
+void log_do_log_daemon(int prio,
+		       __attribute__((unused))void *priv,
+		       const char *fmt,
+		       va_list ap)
 {
 	struct sembuf ops[1];
 
@@ -279,7 +286,10 @@ void log_do_log_daemon(int prio, void *priv, const char *fmt, va_list ap)
 		syslog(LOG_ERR, "semop up failed");
 }
 
-void log_do_log_std(int prio, void *priv, const char *fmt, va_list ap)
+void log_do_log_std(int prio,
+		    __attribute__((unused))void *priv,
+		    const char *fmt,
+		    va_list ap)
 {
 	if (prio == LOG_INFO) {
 		vfprintf(stdout, fmt, ap);
@@ -326,6 +336,7 @@ void log_info(const char *fmt, ...)
 	va_end(ap);
 }
 
+#if 0 /* Unused */
 static void __dump_line(int level, unsigned char *buf, int *cp)
 {
 	char line[16*3+5], *lp = line;
@@ -359,6 +370,7 @@ static void __dump_char(int level, unsigned char *buf, int *cp, int ch)
 
 #define dump_line() __dump_line(level, char_buf, &char_cnt)
 #define dump_char(ch) __dump_char(level, char_buf, &char_cnt, ch)
+#endif /* Unused */
 
 static void log_flush(void)
 {
@@ -474,6 +486,8 @@ void log_close(pid_t pid)
 		return;
 	}
 
-	kill(pid, SIGTERM);
-	waitpid(pid, &status, 0);
+	if (pid > 0) {
+		kill(pid, SIGTERM);
+		waitpid(pid, &status, 0);
+	}
 }
